@@ -1,6 +1,6 @@
 package Net::OAuth::Easy::Roles::Types;
 BEGIN {
-  $Net::OAuth::Easy::Roles::Types::VERSION = '0.001_04';
+  $Net::OAuth::Easy::Roles::Types::VERSION = '0.001_05';
 }
 use Moose::Role;
 use Moose::Util::TypeConstraints;
@@ -8,6 +8,8 @@ use File::Slurp;
 use Data::Validate::URI qw(is_uri);
 require Crypt::OpenSSL::RSA;
 
+# ABSTRACT: Role to tuck away types into
+
 enum 'OAuthProtocol' => qw(1.0 1.0a);
 
 enum 'RequestMethod' => qw(GET POST);
@@ -26,3 +28,28 @@ coerce  SignatureKey =>
 type ValidURI => as Str => where {is_uri($_)};
 
 1;
+
+__END__
+=pod
+
+=head1 NAME
+
+Net::OAuth::Easy::Roles::Types - Role to tuck away types into
+
+=head1 VERSION
+
+version 0.001_05
+
+=head1 AUTHOR
+
+  Ben Hengst <notbenh@cpan.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Ben Hengst.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
+
